/** ClientEventHandler.java.

	Purpose:
		
	Description:
		
	History:
		12:03:30 PM Jul 7, 2014, Created by jumperchen

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */
package org.zkoss.chart;

import java.util.Iterator;
import java.util.Set;

import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;

/**
 * A way to handle a client event listener.
 * 
 * @author jumperchen
 * @since 1.2.0
 */
public interface ClientEventHandler {

	/**
	 * Adds an event listener to this options.
	 * 
	 * @param evtnm
	 *            what event to listen (never null), i.e. onClick, onMouseOver,
	 *            and so on.
	 */
	public void addEventListener(String evtnm,
			EventListener<? extends Event> listener);

	/**
	 * Removes an event listener.
	 * 
	 * @return whether the listener is removed; false if it was never added.
	 */
	public boolean removeEventListener(String evtnm,
			EventListener<? extends Event> listener);

	/**
	 * Returns an iterative collection of the event listeners for the given
	 * event.
	 * <p>
	 * Note: it is OK to invoke {@link #addEventListener} or
	 * {@link #removeEventListener} when iterating through the event listeners
	 * with the returned collection.
	 * <p>
	 * To remove an event listener from the returned iterative collection, you
	 * could invoke {@link Iterable#iterator}'s {@link Iterator#remove}.
	 */
	public Iterable<EventListener<? extends Event>> getEventListeners(
			String evtnm);

	/**
	 * Returns all event names, if any.
	 */
	public Set<String> getEventNames();
}
