package org.zkoss.chart.options3D;

import org.zkoss.chart.Optionable;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * A 3D frame options
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author hanhsu
 */
public class Frame3D extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		back, bottom, side
	}

	/**
	 * Returns the options for the back panel of a {@link Frame3D}
	 * <p>
	 * Default: an instance of {@link BackPanel}.
	 */
	public BackPanel getBack() {
		BackPanel back = (BackPanel) getAttr(Attrs.back);
		if (back == null) {
			back = new BackPanel();
			setBack(back);
		}
		return back;
	}

	/**
	 * Sets the options for the back panel of a {@link Frame3D}
	 */
	public void setBack(BackPanel back) {
		setAttr(Attrs.back, back);
	}
	
	/**
	 * Returns the options for the bottom panel of a {@link Frame3D}
	 * <p>
	 * Default: an instance of {@link BottomPanel}.
	 */
	public BottomPanel getBottom() {
		BottomPanel bottom = (BottomPanel) getAttr(Attrs.bottom);
		if (bottom == null) {
			bottom = new BottomPanel();
			setBottom(bottom);
		}
		return bottom;
	}

	/**
	 * Sets the options for the bottom panel of a {@link Frame3D}
	 */
	public void setBottom(BottomPanel bottom) {
		setAttr(Attrs.bottom, bottom);
	}
	
	/**
	 * Returns the options for the side panel of a {@link Frame3D}
	 * <p>
	 * Default: an instance of {@link SidePanel}.
	 */
	public SidePanel getSide() {
		SidePanel side = (SidePanel) getAttr(Attrs.side);
		if (side == null) {
			side = new SidePanel();
			setSide(side);
		}
		return side;
	}

	/**
	 * Sets the options for the side panel of a {@link Frame3D}
	 */
	public void setSide(SidePanel side) {
		setAttr(Attrs.side, side);
	}
}
